#ifndef HEADER_OCSP_H
#define HEADER_OCSP_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <openssl/x509.h>
#include <openssl/x509v3.h>

#include <ocsp_st.h>
#include <ocsp_req_st.h>
#include <ocsp_resp_st.h>

#define OCSP_RESPONSE_STATUS_SUCCESSFULL          0
#define OCSP_RESPONSE_STATUS_MALFORMEDREQUEST     1
#define OCSP_RESPONSE_STATUS_INTERNALERROR        2
#define OCSP_RESPONSE_STATUS_TRYLATER             3
#define OCSP_RESPONSE_STATUS_SIGREQUIRED          5
#define OCSP_RESPONSE_STATUS_UNAUTHORIZED         6

#define V_OCSP_RESPID_NAME 1
#define V_OCSP_RESPID_KEY  2

#define OCSP_REVOKED_STATUS_NOSTATUS               -1
#define OCSP_REVOKED_STATUS_UNSPECIFIED             0
#define OCSP_REVOKED_STATUS_KEYCOMPROMISE           1
#define OCSP_REVOKED_STATUS_CACOMPROMISE            2
#define OCSP_REVOKED_STATUS_AFFILIATIONCHANGED      3
#define OCSP_REVOKED_STATUS_SUPERSEDED              4
#define OCSP_REVOKED_STATUS_CESSATIONOFOPERATION    5
#define OCSP_REVOKED_STATUS_CERTIFICATEHOLD         6
#define OCSP_REVOKED_STATUS_REMOVEFROMCRL           8

#define V_OCSP_CERTSTATUS_GOOD    0
#define V_OCSP_CERTSTATUS_REVOKED 1
#define V_OCSP_CERTSTATUS_UNKNOWN 2
 
#define PEM_STRING_OCSP_REQUEST	"OCSP REQUEST"
#define PEM_STRING_OCSP_RESPONSE "OCSP RESPONSE"

/*
#define d2i_OCSP_REQUEST_bio(bp,p) (OCSP_REQUEST*)ASN1_d2i_bio((char*(*)()) OCSP_REQUEST_new,(char *(*)())d2i_OCSP_REQUEST, (bp), (unsigned char **)(p) )

#define d2i_OCSP_RESPONSE_bio(bp,p) (OCSP_RESPONSE*)ASN1_d2i_bio((char*(*)()) OCSP_REQUEST_new,(char *(*)())d2i_OCSP_RESPONSE, (bp), (unsigned char **)(p))

#define	PEM_read_bio_OCSP_REQUEST(bp,x,cb) (OCSP_REQUEST *)PEM_ASN1_read_bio( (char *(*)())d2i_OCSP_REQUEST,PEM_STRING_OCSP_REQUEST,bp,(char **)x,cb, NULL)

#define	PEM_read_bio_OCSP_RESPONSE(bp,x,cb)(OCSP_RESPONSE *)PEM_ASN1_read_bio( (char *(*)())d2i_OCSP_RESPONSE,PEM_STRING_OCSP_RESPONSE,bp,(char **)x,cb, NULL)

#define PEM_write_bio_OCSP_REQUEST(bp,o) PEM_ASN1_write_bio((int (*)())i2d_OCSP_REQUEST,PEM_STRING_OCSP_REQUEST, bp,(char *)o, NULL,NULL,0,NULL,NULL)

#define PEM_write_bio_OCSP_RESPONSE(bp,o) PEM_ASN1_write_bio((int (*)())i2d_OCSP_RESPONSE,PEM_STRING_OCSP_RESPONSE, bp,(char *)o, NULL,NULL,0,NULL,NULL)

#define i2d_OCSP_RESPONSE_bio(bp,o) ASN1_i2d_bio(i2d_OCSP_RESPONSE,bp, (unsigned char *)o )

#define i2d_OCSP_REQUEST_bio(bp,o) ASN1_i2d_bio(i2d_OCSP_REQUEST,bp, (unsigned char *)o )

#define OCSP_REQUEST_sign(o,pkey,md) ASN1_sign((int(*)())i2d_OCSP_REQINFO, o->optionalSignature->signatureAlgorithm,NULL, o->optionalSignature->signature,(char *)o->tbsRequest,pkey,md)

#define OCSP_BASICRESP_sign(o,pkey,md) ASN1_sign((int(*)())i2d_OCSP_RESPDATA,o->signatureAlgorithm,NULL, o->signature,(char *)o->tbsResponseData,pkey,md)

#define OCSP_REQUEST_verify(a,r) ASN1_verify((int (*)())i2d_OCSP_REQINFO, a->optionalSignature->signatureAlgorithm, a->optionalSignature->signature,(char *)a->tbsRequest,r)

#define OCSP_BASICRESP_verify(a,r) ASN1_verify((int (*)())i2d_OCSP_RESPDATA, a->signatureAlgorithm,a->signature,(char *)a->tbsResponseData,r)

#define ASN1_BIT_STRING_digest(data,type,md,len) ASN1_digest((int (*)())i2d_ASN1_BIT_STRING,type,(char *)data,md,len)

#define OCSP_CERTID_dup(cid) (OCSP_CERTID*)ASN1_dup((int(*)())i2d_OCSP_CERTID, (char *(*)())d2i_OCSP_CERTID,(char *)(cid))

#define OCSP_CERTSTATUS_dup(cs) (OCSP_CERTSTATUS*)ASN1_dup((int(*)())i2d_OCSP_CERTSTATUS, (char *(*)())d2i_OCSP_CERTSTATUS,(char *)(cs))
*/

#ifndef NOPROTO

/* Now Implemented */
OCSP_CERTID *OCSP_cert_id_new(EVP_MD *dgst, 
			      X509_NAME *issuerName, 
			      ASN1_BIT_STRING* issuerKey, 
			      ASN1_INTEGER *serialNumber);

/* Now Implemented */
OCSP_CERTID *OCSP_cert_id_new(EVP_MD *dgst, 
			      X509_NAME *issuerName, 
			      ASN1_BIT_STRING* issuerKey, 
			      ASN1_INTEGER *serialNumber);

/* Now Implemented */
OCSP_CERTSTATUS *OCSP_cert_status_new(int status, int reason, char *time);

/* Now Implemented */
OCSP_REQUEST *OCSP_request_new(X509_NAME* name, STACK *extensions);

/* Now Implemented */
int OCSP_request_add(OCSP_REQUEST    *req,
		     OCSP_CERTID     *cid,
		     STACK           *extensions);

/* Now Implemented */
int OCSP_request_sign(OCSP_REQUEST *req,
		      EVP_PKEY     *key,
		      EVP_MD       *dgst,
		      STACK        *certs);

int OCSP_request_verify(OCSP_REQUEST *req, EVP_PKEY *pkey);

OCSP_BASICRESP *OCSP_basic_response_new(int tag,X509* cert,STACK *extensions);

int OCSP_basic_response_add(OCSP_BASICRESP  *rsp,
			    OCSP_CERTID     *cid,
			    OCSP_CERTSTATUS *cst,
			    char            *thisUpdate,
			    char            *nextUpdate,
			    STACK           *extensions);

int OCSP_basic_response_sign(OCSP_BASICRESP *brsp, 
			     EVP_PKEY       *key,
			     EVP_MD         *dgst,
			     STACK          *certs);

int OCSP_response_verify(OCSP_RESPONSE *rsp, EVP_PKEY *pkey);

int OCSP_basic_response_verify(OCSP_BASICRESP *rsp, EVP_PKEY *pkey);

OCSP_RESPONSE *OCSP_response_new(int status,
				 int nid,
				 int (*i2d)(),
				 char *data);

ASN1_STRING *ASN1_STRING_encode(ASN1_STRING *s, int (*i2d)(), 
				char *data, STACK *sk);

X509_EXTENSION *OCSP_nonce_new(void *p, unsigned int len);

X509_EXTENSION *OCSP_crlref_new(char *url, long *n, char *time);

X509_EXTENSION *OCSP_accept_responses_new(char **oids);

X509_EXTENSION *OCSP_archive_cutoff_new(char* time);

X509_EXTENSION *OCSP_url_svcloc_new(X509_NAME* issuer, char **urls);

OCSP_SINGLERESP *OCSP_SINGLERESP_new();
void OCSP_SINGLERESP_free(OCSP_SINGLERESP *a);
int i2d_OCSP_SINGLERESP(OCSP_SINGLERESP *a, unsigned char **pp);
OCSP_SINGLERESP *d2i_OCSP_SINGLERESP(OCSP_SINGLERESP **a, unsigned char **pp, long length);
int i2a_OCSP_SINGLERESP(BIO *bp, OCSP_SINGLERESP* a);

OCSP_CERTSTATUS *OCSP_CERTSTATUS_new();
void OCSP_CERTSTATUS_free(OCSP_CERTSTATUS *a);
int i2d_OCSP_CERTSTATUS(OCSP_CERTSTATUS *a, unsigned char **pp);
OCSP_CERTSTATUS *d2i_OCSP_CERTSTATUS(OCSP_CERTSTATUS **a, unsigned char **pp, long length);
int i2a_OCSP_CERTSTATUS(BIO *bp, OCSP_CERTSTATUS* a);

OCSP_REVOKEDINFO *OCSP_REVOKEDINFO_new();
void OCSP_REVOKEDINFO_free(OCSP_REVOKEDINFO *a);
int i2d_OCSP_REVOKEDINFO(OCSP_REVOKEDINFO *a, unsigned char **pp);
OCSP_REVOKEDINFO *d2i_OCSP_REVOKEDINFO(OCSP_REVOKEDINFO **a, unsigned char **pp, long length);
int i2a_OCSP_REVOKEDINFO(BIO *bp, OCSP_REVOKEDINFO* a);

OCSP_BASICRESP *OCSP_BASICRESP_new();
void OCSP_BASICRESP_free(OCSP_BASICRESP *a);
int i2d_OCSP_BASICRESP(OCSP_BASICRESP *a, unsigned char **pp);
OCSP_BASICRESP *d2i_OCSP_BASICRESP(OCSP_BASICRESP **a, unsigned char **pp, long length);
int i2a_OCSP_BASICRESP(BIO *bp, OCSP_BASICRESP* a);

OCSP_RESPDATA *OCSP_RESPDATA_new();
void OCSP_RESPDATA_free(OCSP_RESPDATA *a);
int i2d_OCSP_RESPDATA(OCSP_RESPDATA *a, unsigned char **pp);
OCSP_RESPDATA *d2i_OCSP_RESPDATA(OCSP_RESPDATA **a, unsigned char **pp, long length);
int i2a_OCSP_RESPDATA(BIO *bp, OCSP_RESPDATA* a);

OCSP_RESPID *OCSP_RESPID_new();
void OCSP_RESPID_free(OCSP_RESPID *a);
int i2d_OCSP_RESPID(OCSP_RESPID *a, unsigned char **pp);
OCSP_RESPID *d2i_OCSP_RESPID(OCSP_RESPID **a, unsigned char **pp, long length);
int i2a_OCSP_RESPID(BIO *bp, OCSP_RESPID* a);

OCSP_RESPONSE *OCSP_RESPONSE_new();
void OCSP_RESPONSE_free(OCSP_RESPONSE *a);
int i2d_OCSP_RESPONSE(OCSP_RESPONSE *a, unsigned char **pp);
OCSP_RESPONSE *d2i_OCSP_RESPONSE(OCSP_RESPONSE **a, unsigned char **pp, long length);
int i2a_OCSP_RESPONSE(BIO *bp, OCSP_RESPONSE* a);
int OCSP_RESPONSE_print(BIO *bp, OCSP_RESPONSE* a);

OCSP_RESPBYTES *OCSP_RESPBYTES_new();
void OCSP_RESPBYTES_free(OCSP_RESPBYTES *a);
int i2d_OCSP_RESPBYTES(OCSP_RESPBYTES *a, unsigned char **pp);
OCSP_RESPBYTES *d2i_OCSP_RESPBYTES(OCSP_RESPBYTES **a, unsigned char **pp, long length);
int i2a_OCSP_RESPBYTES(BIO *bp, OCSP_RESPBYTES* a);

OCSP_ONEREQ *OCSP_ONEREQ_new();
void OCSP_ONEREQ_free(OCSP_ONEREQ *a);
int i2d_OCSP_ONEREQ(OCSP_ONEREQ *a, unsigned char **pp);
OCSP_ONEREQ *d2i_OCSP_ONEREQ(OCSP_ONEREQ **a, unsigned char **pp, long length);
int i2a_OCSP_ONEREQ(BIO *bp, OCSP_ONEREQ* a);

OCSP_CERTID *OCSP_CERTID_new();
void OCSP_CERTID_free(OCSP_CERTID *a);
int i2d_OCSP_CERTID(OCSP_CERTID *a, unsigned char **pp);
OCSP_CERTID *d2i_OCSP_CERTID(OCSP_CERTID **a, unsigned char **pp, long length);
int i2a_OCSP_CERTID(BIO *bp, OCSP_CERTID* a);

OCSP_REQUEST *OCSP_REQUEST_new();
void OCSP_REQUEST_free(OCSP_REQUEST *a);
int i2d_OCSP_REQUEST(OCSP_REQUEST *a, unsigned char **pp);
OCSP_REQUEST *d2i_OCSP_REQUEST(OCSP_REQUEST **a, unsigned char **pp, long length);
int i2a_OCSP_REQUEST(BIO *bp, OCSP_REQUEST* a);
int OCSP_REQUEST_print(BIO *bp, OCSP_REQUEST* a);

OCSP_SIGNATURE *OCSP_SIGNATURE_new();
/* void OCSP_SIGNATURE_free(OCSP_SIGNATURE *a); */
int i2d_OCSP_SIGNATURE(OCSP_SIGNATURE *a, unsigned char **pp);
OCSP_SIGNATURE *d2i_OCSP_SIGNATURE(OCSP_SIGNATURE **a, unsigned char **pp, long length);
int i2a_OCSP_SIGNATURE(BIO *bp, OCSP_SIGNATURE* a);

OCSP_REQINFO *OCSP_REQINFO_new();
void OCSP_REQINFO_free(OCSP_REQINFO *a);
int i2d_OCSP_REQINFO(OCSP_REQINFO *a, unsigned char **pp);
OCSP_REQINFO *d2i_OCSP_REQINFO(OCSP_REQINFO **a, unsigned char **pp, long length);
int i2a_OCSP_REQINFO(BIO *bp, OCSP_REQINFO* a);

OCSP_CRLID *OCSP_CRLID_new();
void OCSP_CRLID_free(OCSP_CRLID *a);
int i2d_OCSP_CRLID(OCSP_CRLID *a, unsigned char **pp);
OCSP_CRLID *d2i_OCSP_CRLID(OCSP_CRLID **a, unsigned char **pp, long length);
int i2a_OCSP_CRLID(BIO *bp, OCSP_CRLID* a);
int OCSP_CRLID_print(BIO *bp, OCSP_CRLID *a, int ind);

OCSP_SERVICELOC *OCSP_SERVICELOC_new();
void OCSP_SERVICELOC_free(OCSP_SERVICELOC *a);
int i2d_OCSP_SERVICELOC(OCSP_SERVICELOC *a, unsigned char **pp);
OCSP_SERVICELOC *d2i_OCSP_SERVICELOC(OCSP_SERVICELOC **a, unsigned char **pp, long length);
int i2a_OCSP_SERVICELOC(BIO *bp, OCSP_SERVICELOC* a);
int OCSP_SERVICELOC_print(BIO *bp, OCSP_SERVICELOC* a, int ind);

int OCSP_extensions_print(BIO *bp, STACK *sk, char *title);
int OCSP_extension_print(BIO *bp, X509_EXTENSION *x, int ind);

void ERR_load_OCSP_strings();

#else

OCSP_CERTID *OCSP_cert_id_new();

OCSP_CERTID *OCSP_cert_id_new();

OCSP_CERTSTATUS *OCSP_cert_status_new();

OCSP_REQUEST *OCSP_request_new();

int OCSP_request_add();

int OCSP_request_sign();

int OCSP_request_verify();

OCSP_BASICRESP *OCSP_basic_response_new();

int OCSP_basic_response_add();

int OCSP_basic_response_sign();

int OCSP_response_verify();

int OCSP_basic_response_verify();

OCSP_RESPONSE *OCSP_response_new();

ASN1_STRING *ASN1_STRING_encode(); 

X509_EXTENSION *OCSP_nonce_new();

X509_EXTENSION *OCSP_crlref_new();

X509_EXTENSION *OCSP_accept_responses_new();

X509_EXTENSION *OCSP_archive_cutoff_new();

X509_EXTENSION *OCSP_url_svcloc_new();

OCSP_SINGLERESP *OCSP_SINGLERESP_new();
void OCSP_SINGLERESP_free();
int i2d_OCSP_SINGLERESP();
OCSP_SINGLERESP *d2i_OCSP_SINGLERESP();
int i2a_OCSP_SINGLERESP();

OCSP_CERTSTATUS *OCSP_CERTSTATUS_new();
void OCSP_CERTSTATUS_free();
int i2d_OCSP_CERTSTATUS();
OCSP_CERTSTATUS *d2i_OCSP_CERTSTATUS();
int i2a_OCSP_CERTSTATUS();

OCSP_REVOKEDINFO *OCSP_REVOKEDINFO_new();
void OCSP_REVOKEDINFO_free();
int i2d_OCSP_REVOKEDINFO();
OCSP_REVOKEDINFO *d2i_OCSP_REVOKEDINFO();
int i2a_OCSP_REVOKEDINFO();

OCSP_BASICRESP *OCSP_BASICRESP_new();
void OCSP_BASICRESP_free();
int i2d_OCSP_BASICRESP();
OCSP_BASICRESP *d2i_OCSP_BASICRESP();
int i2a_OCSP_BASICRESP();

OCSP_RESPDATA *OCSP_RESPDATA_new();
void OCSP_RESPDATA_free();
int i2d_OCSP_RESPDATA();
OCSP_RESPDATA *d2i_OCSP_RESPDATA();
int i2a_OCSP_RESPDATA();

OCSP_RESPID *OCSP_RESPID_new();
void OCSP_RESPID_free();
int i2d_OCSP_RESPID();
OCSP_RESPID *d2i_OCSP_RESPID();
int i2a_OCSP_RESPID();

OCSP_RESPONSE *OCSP_RESPONSE_new();
void OCSP_RESPONSE_free();
int i2d_OCSP_RESPONSE();
OCSP_RESPONSE *d2i_OCSP_RESPONSE();
int i2a_OCSP_RESPONSE();
int OCSP_RESPONSE_print();

OCSP_RESPBYTES *OCSP_RESPBYTES_new();
void OCSP_RESPBYTES_free();
int i2d_OCSP_RESPBYTES();
OCSP_RESPBYTES *d2i_OCSP_RESPBYTES();
int i2a_OCSP_RESPBYTES();

OCSP_ONEREQ *OCSP_ONEREQ_new();
void OCSP_ONEREQ_free();
int i2d_OCSP_ONEREQ();
OCSP_ONEREQ *d2i_OCSP_ONEREQ();
int i2a_OCSP_ONEREQ();

OCSP_CERTID *OCSP_CERTID_new();
void OCSP_CERTID_free();
int i2d_OCSP_CERTID();
OCSP_CERTID *d2i_OCSP_CERTID();
int i2a_OCSP_CERTID();

OCSP_REQUEST *OCSP_REQUEST_new();
void OCSP_REQUEST_free();
int i2d_OCSP_REQUEST();
OCSP_REQUEST *d2i_OCSP_REQUEST();
int i2a_OCSP_REQUEST();
int OCSP_REQUEST_print();

OCSP_SIGNATURE *OCSP_SIGNATURE_new();
void OCSP_SIGNATURE_free();
int i2d_OCSP_SIGNATURE();
OCSP_SIGNATURE *d2i_OCSP_SIGNATURE();
int i2a_OCSP_SIGNATURE();

OCSP_REQINFO *OCSP_REQINFO_new();
void OCSP_REQINFO_free();
int i2d_OCSP_REQINFO();
OCSP_REQINFO *d2i_OCSP_REQINFO();
int i2a_OCSP_REQINFO();

OCSP_CRLID *OCSP_CRLID_new();
void OCSP_CRLID_free();
int i2d_OCSP_CRLID();
OCSP_CRLID *d2i_OCSP_CRLID();
int i2a_OCSP_CRLID();
int OCSP_CRLID_print();

OCSP_SERVICELOC *OCSP_SERVICELOC_new();
void OCSP_SERVICELOC_free();
int i2d_OCSP_SERVICELOC();
OCSP_SERVICELOC *d2i_OCSP_SERVICELOC();
int i2a_OCSP_SERVICELOC();
int OCSP_SERVICELOC_print();

int OCSP_extensions_print();
int OCSP_extension_print();

void ERR_load_OCSP_strings();
#endif

/* BEGIN ERROR CODES */
/* Error codes for the OCSP functions. */

/* Function codes. */
#define OCSP_F_BASIC_RESPONSE_NEW			 105
#define OCSP_F_BASIC_RESPONSE_VERIFY			 115
#define OCSP_F_CERT_ID_NEW				 100
#define OCSP_F_CERT_STATUS_NEW				 101
#define OCSP_F_REQUEST_VERIFY				 116
#define OCSP_F_RESPONSE_VERIFY				 114

/* Reason codes. */
#define OCSP_R_BAD_TAG					 103
#define OCSP_R_DIGEST_ERR				 102
#define OCSP_R_NO_CERTIFICATE				 106
#define OCSP_R_NO_PUBLIC_KEY				 107
#define OCSP_R_NO_RESPONSE_DATA				 104
#define OCSP_R_NO_SIGNATURE				 105
#define OCSP_R_REVOKED_NO_TIME				 100
#define OCSP_R_UNKNOWN_NID				 101
 
#ifdef  __cplusplus
}
#endif
#endif

char *BUILD_INFO[] = {
        SVERSION,
	NULL
};

