#!/usr/bin/perl

$|=1;

use OpenCA::OpenSSL;
use OpenCA::X509;
use OpenCA::DB;

my $VER = '0.2.01';

print "\nRequest Import - Version $VER\n";
print "(c) 1999/2000 by Massimiliano Pala and OpenCA Team\n\n";

my $file      =   $ARGV[0];
my $opencaDir = ( $ARGV[1] or "/usr/local/OpenCA" );
my $dataType  = ( $ARGV[2] or "PENDING_REQUEST" );
my $sslPath   = ( $ARGV[3] or "/usr/bin/openssl" );

my $data;

if( (not $ARGV[0]) or (not $ARGV[1]) ) {
	usage();
}

print " * Initializing crypto shell ... ";
my $openssl = new OpenCA::OpenSSL ( SHELL => $sslPath );

$openssl->setParams( CONFIG=>"$opencaDir/stuff/openssl.cnf",
		    STDERR=>"/dev/null" );

if( not $openssl ) {
	print "Error!\n\n";
	exit 1;
} else {
	print "Ok.\n";
}

print " * Initializing dB object ... ";
my $db = new OpenCA::DB( SHELL=>$openssl, DB_DIR=>"$opencaDir/dB" );

if( not $db ) {
        print "Error!\n\n";
        exit 1;
} else {
	print "Ok.\n\n";
}

open( FD, "<$file" ) or die "Cannot load $file!";
while ( not eof (FD) ) {
	$data .= <FD>;
}
close( FD );

if( -e "$srcDir/$file.sig" ) {
	open( FD, "<$srcDir/$file.sig" ) or die "Error!";
	while ( not eof( FD ) ) {
		$data .= <FD>;
	}
	close( FD );
};

my $REQ = new OpenCA::REQ( DATA=>$data,
			   SHELL=>$openssl );

die "Cannot Parse Request!" if ( not $REQ );

print " * New Request Data:\n";
print "   Type:     " . $REQ->getParsed()->{TYPE} . "\n";
print "   C-Name:   " . $REQ->getParsed()->{CN} . "\n";
print "   E-Mail:   " . $REQ->getParsed()->{EMAIL} . "\n";
print "   Approved: " . $REQ->getParsed()->{APPROVED} . "\n\n";

print "Storing Request to dB ... ";
if( not $db->storeItem( DATATYPE=>$dataType,
			DATA=>$REQ->{req} ) ) {
  	print "Error!\n\n";
   	next;
} else {
	print "Ok.\n\n";
}

exit 0; 

sub usage {

	print "     Usage: addReqDB.pl reqfile [openca-dir] [openssl-path]\n\n";
	print "  reqfile      - PEM request file\n";
	print "  type         - Request dataType (def. PENDING_REQUEST)\n";
	print "  openca-dir   - OpenCA directory (def. /usr/local/OpenCA)\n";
	print "  openssl-path - Path to the openssl command (def. $sslPath)\n";
	print "\n";

	exit 0;
}
